/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Tinderbox Status Summary.
 *
 * The Initial Developer of the Original Code is
 * Myk Melez <myk@melez.com>.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Alexander J. Vincent <ajvincent@gmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// This file was completely rewritten in June, 2005 by Alexander J. Vincent.
if (!Cc)
  var Cc = Components.classes;
if (!Ci)
  var Ci = Components.interfaces;

var gXMLHttpRequest;

// The order in this list somewhat matters, as the first item is the one
// that gets loaded when you click on the statusbar icon.
var gTinderboxList  = [];
var gTinderboxListUpdateRequired = true;
var gTinderboxIndex = -1;
var gTinderboxText  = "";
var gTinderboxURIIndex = 0;
var gTinderboxStatus = null;

var gPrefService = Cc["@mozilla.org/preferences-service;1"]
  .getService(Ci.nsIPrefService);
var gPrefBranch = gPrefService.getBranch("extensions.tinderstatus.");

var tinderstatusPrefObserver = 
{
  register: function()
  {
    if (gPrefBranch instanceof Ci.nsIPrefBranch2) {
      gPrefBranch.addObserver("", this, false);
    }
  },

  unregister: function()
  {
    if (gPrefBranch instanceof Ci.nsIPrefBranch2) {
      gPrefBranch.removeObserver("", this);
    }
  },

  observe: function(aSubject, aTopic, aData)
  {
    if (aTopic != "nsPref:changed")
      return;

    switch (aData) {
      case "watchList":
        gTinderboxListUpdateRequired = true;
        refreshTinderboxenStatus();
        break;
    }
  }
};

function updateCurrentTinderboxList() {
  try {
    gTinderboxList = gPrefBranch.getComplexValue("watchList",
                                                 Ci.nsISupportsString).data;
    gTinderboxList = gTinderboxList.split(",");

    if (gTinderboxList.length == 1 && gTinderboxList[0] == "")
      gTinderboxList = [];
  }
  catch (ex) {
    // Set to the default list
    gTinderboxList  = ["Firefox",
                       "SeaMonkey",
                       "Thunderbird",
                       "XULRunner",
                       "Camino"];
  }
  gTinderboxListUpdateRequired = false;
}

/**
 * Load into the browser a current tinderbox page.
 */
function loadTinderboxURI(evt) {
  var tinderboxToLoad;
  try {
    tinderboxToLoad = gPrefBranch.getComplexValue("gotoTree",
                                                  Ci.nsISupportsString).data;
  }
  catch (ex) {
    // If we can't get the pref, set to the default
    tinderboxToLoad = "Firefox";
  }

  if (tinderboxToLoad != "") {
    var url = "http://tinderbox.mozilla.org/" + tinderboxToLoad + "/";
    if (!handleLinkClick(evt, url, null))
      loadURI(url);
  }
}

function refreshTinderboxenStatus() {
  // Check we aren't currently doing an update
  if (gTinderboxIndex != -1)
    return;

  if (gTinderboxListUpdateRequired)
    updateCurrentTinderboxList();

  gTinderboxURIIndex = 0;
  getTinderboxStatus();
}

/**
 * Request a tinderbox summary from one tree.
 */
function getTinderboxStatus() {
  // If no items selected, just show nothing.
  if (gTinderboxList.length == 0) {
    document.getElementById('tinderstatus-box')
            .setAttribute("buildstatus", "none");

    setTinderboxStatus("No tinderboxes selected.");
    gTinderboxURIIndex = 0;
    return;
  }

  gTinderboxIndex++;
  if (gTinderboxIndex == gTinderboxList.length) {
    return;
  }
  var url = "http://tinderbox.mozilla.org/";
  url += gTinderboxList[gTinderboxIndex];
  url += "/quickparse.txt";

  // Oh, how I wish we didn't have to reinitialize every time.
  gXMLHttpRequest = new XMLHttpRequest();
  gXMLHttpRequest.onload = displayTinderboxStatus;
  gXMLHttpRequest.open("GET", url);
  gXMLHttpRequest.send(null);
}

/**
 * Indicate via tooltip tinderboxes which show failures.
 */
function fillTinderboxStatus(aRegExp) {
  var results = gTinderboxText.match(aRegExp);

  while (gTinderboxStatus.firstChild) {
    gTinderboxStatus.removeChild(gTinderboxStatus.firstChild);
  }

  for (var i = 0; i < results.length; i++) {
    var node = document.createElement("description");
    node.setAttribute("value", results[i]);
    gTinderboxStatus.appendChild(node);
  }
}

/**
 * Indicate via tooltip a status message (usually success).
 */
function setTinderboxStatus(aMessage) {
  while (gTinderboxStatus.firstChild != gTinderboxStatus.lastChild) {
    gTinderboxStatus.removeChild(gTinderboxStatus.firstChild);
  }

  gTinderboxStatus.firstChild.setAttribute("value", aMessage);
}

/**
 * Handle response from tinderbox server, and update status bar.
 */
function displayTinderboxStatus()
{
  var text = gXMLHttpRequest.responseText + "\n";
  gTinderboxText += text;

  // Note which tinderbox fails.
  if ((gTinderboxURIIndex == 0) &&
      ((text.indexOf("busted") != -1) || (text.indexOf("testfailed") != -1))) {
    gTinderboxURIIndex = gTinderboxIndex;
  }

  // Do we need to loop around again?
  if (gTinderboxIndex < (gTinderboxList.length - 1)) {
    getTinderboxStatus();
    return;
  }

  // Change the background color of the box depending to reflect build status.
  var box = document.getElementById('tinderstatus-box');
  var re;
  if (gTinderboxText.indexOf("busted") != -1) {
    re = /.*(busted|testfailed)/g;
    fillTinderboxStatus(re);
    box.setAttribute("buildstatus", "busted");
  }
  else if (gTinderboxText.indexOf("testfailed") != -1) {
    re = /.*testfailed/g;
    fillTinderboxStatus(re);
    box.setAttribute("buildstatus", "testfailed");
  }
  else if (gTinderboxText.indexOf("success") != -1) {
    setTinderboxStatus("All tinderboxes are green.");
    box.setAttribute("buildstatus", "success");
    gTinderboxURIIndex = 0;
  }
  else {
    setTinderboxStatus("Unknown tinderbox status.");
    box.setAttribute("buildstatus", "none");
    gTinderboxURIIndex = 0;
  }

  // Change the icon superimposed on the box to reflect tree status.
  var icon = document.getElementById('tinderstatus-icon');
  if (gTinderboxText.indexOf("closed") != -1)
    icon.setAttribute("treestatus", "closed");
  else if (gTinderboxText.indexOf("open") != -1)
    icon.setAttribute("treestatus", "open");
  else
    icon.setAttribute("treestatus", "none");

  gTinderboxIndex = -1;
  gTinderboxText = "";

  // If we have just done an update, and the an update request is signalled
  // due to change of pref, run the update again
  if (gTinderboxListUpdateRequired)
    refreshTinderboxenStatus();
}

tinderstatusPrefObserver.register();

// Check the status a second after the browser loads.
window.addEventListener("load", function initTinderboxStatus(evt) {
  gTinderboxStatus = document.getElementById("tinderstatus-tooltip").firstChild;
  window.setTimeout(refreshTinderboxenStatus, 1000);

  // Wait one minute before trying again.
  window.setInterval(refreshTinderboxenStatus, 60000);
}, false);
